/**
 * Copyright(c) Live2D Inc. All rights reserved.
 *
 * Use of this source code is governed by the Live2D Open Software license
 * that can be found at https://www.live2d.com/eula/live2d-open-software-license-agreement_en.html.
 */

#pragma once

#include <string>
#include <gxm.h>
#include <touch.h>
#include "Type/csmVector.hpp"
#include "LAppAllocator.hpp"
#include "LAppView.hpp"

#ifdef _SCE_TARGET_OS_PSP2

extern const SceGxmProgram _binary_fragmentShader_gxp_start;
extern const SceGxmProgram _binary_vertexShader_gxp_start;

static const SceGxmProgram *fragmentShader = &_binary_fragmentShader_gxp_start;
static const SceGxmProgram *vertexShader = &_binary_vertexShader_gxp_start;

#endif

class LAppView;
class LAppTextureManager;

/**
* @brief   アプリケーションクラス。
*   Cubism SDK の管理を行う。
*/
class LAppDelegate
{
public:
    /**
    * @brief   クラスのインスタンス（シングルトン）を返す。<br>
    *           インスタンスが生成されていない場合は内部でインスタンを生成する。
    *
    * @return  クラスのインスタンス
    */
    static LAppDelegate* GetInstance();

    /**
    * @brief   クラスのインスタンス（シングルトン）を解放する。
    *
    */
    static void ReleaseInstance();

    /**
    * @brief   APPに必要なものを初期化する。
    */
    bool Initialize();

    /**
    * @brief   解放する。
    */
    void Release();

    /**
    * @brief   実行処理。
    */
    void Run();

    /**
    * @brief シェーダーを登録する。
    */
    void CreateShader(LAppView *view);

    /**
    * @brief   View情報を取得する。
    */
    LAppView* GetView() { return _view; }

    /**
    * @brief   アプリケーションを終了するかどうか。
    */
    bool GetIsEnd() { return _isEnd; }

    /**
    * @brief   アプリケーションを終了する。
    */
    void AppEnd() { _isEnd = true; }

    /**
     * @brief   ルートディレクトリを設定する。
     */
    void SetRootDirectory();

    /**
     * @brief   ルートディレクトリを取得する。
     */
    std::string GetRootDirectory(){ return _rootDirectory;}

    /**
     * @brief   テクスチャマネージャーを取得する。
     */
    LAppTextureManager* GetTextureManager() { return _textureManager; }

private:
    /**
    * @brief   コンストラクタ
    */
    LAppDelegate();

    /**
    * @brief   デストラクタ
    */
    ~LAppDelegate();

    /**
    * @brief   Cubism SDK の初期化
    */
    void InitializeCubism();

    void CheckControls();

    /**
     * @brief   文字列を指定の文字で切り分ける
     */
    Csm::csmVector<std::string> Split(const std::string& baseString, char delim);

    LAppAllocator _cubismAllocator;              ///< Cubism SDK Allocator
    Csm::CubismFramework::Option _cubismOption;  ///< Cubism SDK Option
    LAppView* _view;                             ///< View情報

    SceTouchData _tdOld;

    bool _captured;                              ///< クリックしているか
    float _mouseX;                               ///< マウスX座標
    float _mouseY;                               ///< マウスY座標
    bool _isEnd;                                 ///< APP終了しているか
    LAppTextureManager* _textureManager;         ///< テクスチャマネージャー
    std::string _rootDirectory; ///< ルートディレクトリ

    int _windowWidth;                            ///< Initialize関数で設定したウィンドウ幅
    int _windowHeight;                           ///< Initialize関数で設定したウィンドウ高さ
};
