﻿#pragma once

#include <string>
#include <gxm.h>
#include <Type/csmVector.hpp>
#include "CubismFramework.hpp"

#define ALIGN(x, a)	(((x) + ((a) - 1)) & ~((a) - 1))

class LAppGXM
{
public:

	SceGxmDeviceMemInfo *vertexStorage;
	SceGxmDeviceMemInfo *indexStorage;

	SceGxmContext *immContext;
	SceGxmRenderTarget *renderTarget;
	void *displayBufferData[2];
	SceGxmSyncObject *displayBufferSync[2];
	SceGxmColorSurface displayColorSurface[2];
	SceGxmDepthStencilSurface depthStencilSurface;
	SceGxmShaderPatcher *shaderPatcher;
	SceGxmMultisampleMode msaa;

	struct GXMTexture
	{
	public:
		SceGxmTexture gxmTexture;
		SceGxmDeviceMemInfo *memData;
	};

	struct InitParams
	{
	public:
		uint32_t paramBufferSize;
		uint32_t vdmRingBufferSize;
		uint32_t vertexRingBufferSize;
		uint32_t fragmentRingBufferSize;
		uint32_t fragmentUsseRingBufferSize;
		SceGxmMultisampleMode msaa;
	};

	LAppGXM(InitParams *params);
	virtual ~LAppGXM();

	static LAppGXM *GetInstance();
	static void ReleaseInstance();

	SceGxmMultisampleMode GetMsaa() { return msaa; }
	void StartScene();
	void EndRendering();
	void EndScene();
	GXMTexture *LoadGxtTexture(std::string fileName, int index);
	void UnloadGxtTexture(LAppGXM::GXMTexture *texture);
	int GetDisplayIndex();
	void WaitRenderingDone();

private:

	struct DisplayCallbackArg
	{
		void* address;
	};

	SceGxmValidRegion _validRegion;

	SceGxmDeviceMemInfo *_vdmRingBufferMem;
	SceGxmDeviceMemInfo *_vertexRingBufferMem;
	SceGxmDeviceMemInfo *_fragmentRingBufferMem;
	SceGxmDeviceMemInfo *_fragmentUsseRingBufferMem;
	SceGxmDeviceMemInfo *_depthBufferMem;
	SceGxmDeviceMemInfo *_stencilBufferMem;
	SceGxmDeviceMemInfo *_patcherBufferMem;
	SceGxmDeviceMemInfo *_patcherVertexUsseMem;
	SceGxmDeviceMemInfo *_patcherFragmentUsseMem;
	SceGxmDeviceMemInfo *_displayBufferMem[2];

	uint32_t _bufferIndex;

	static void DisplayCallback(const void *callbackData);
	static void *PatcherHostAlloc(void *user_data, uint32_t size);
	static void PatcherHostFree(void *user_data, void *mem);
};
