/**
 * Copyright(c) Live2D Inc. All rights reserved.
 *
 * Use of this source code is governed by the Live2D Open Software license
 * that can be found at https://www.live2d.com/eula/live2d-open-software-license-agreement_en.html.
 */

#include "LAppTextureManager.hpp"
#include <iostream>
#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-function"
#endif
#if defined(__clang__)
#pragma clang diagnostic pop
#endif
#include "LAppPal.hpp"
#include "LAppGXM.hpp"

LAppTextureManager::LAppTextureManager()
{
}

LAppTextureManager::~LAppTextureManager()
{
    ReleaseTextures();
}

LAppTextureManager::TextureInfo* LAppTextureManager::CreateTextureFromGxtFile(std::string fileName)
{
    //search loaded texture already.
    for (Csm::csmUint32 i = 0; i < _textures.GetSize(); i++)
    {
        if (_textures[i]->fileName == fileName)
        {
            return _textures[i];
        }
    }

    LAppGXM::GXMTexture *texture = LAppGXM::GetInstance()->LoadGxtTexture(fileName, 0);

    {

#ifdef PREMULTIPLIED_ALPHA_ENABLE
        unsigned int* fourBytes = reinterpret_cast<unsigned int*>(texture->memData->mappedBase);
        for (int i = 0; i < width * height; i++)
        {
            unsigned char* p = reinterpret_cast<unsigned char*>(texture->memData->mappedBase) + i * 4;
            fourBytes[i] = Premultiply(p[0], p[1], p[2], p[3]);
        }
#endif
    }

    sceGxmTextureSetMinFilter(&texture->gxmTexture ,SCE_GXM_TEXTURE_FILTER_LINEAR);
    sceGxmTextureSetMagFilter(&texture->gxmTexture, SCE_GXM_TEXTURE_FILTER_LINEAR);

    LAppTextureManager::TextureInfo* textureInfo = new LAppTextureManager::TextureInfo();
    if (textureInfo != NULL)
    {
        textureInfo->texture = texture;
        textureInfo->fileName = fileName;
        textureInfo->width = sceGxmTextureGetWidth(&texture->gxmTexture);
        textureInfo->height = sceGxmTextureGetHeight(&texture->gxmTexture);
        _textures.PushBack(textureInfo);
    }

    return textureInfo;

}

void LAppTextureManager::ReleaseTextures()
{
    for (Csm::csmUint32 i = 0; i < _textures.GetSize(); i++)
    {
        LAppGXM::GetInstance()->UnloadGxtTexture(_textures[i]->texture);
        delete _textures[i];
    }

    _textures.Clear();
}

void LAppTextureManager::ReleaseTexture(LAppGXM::GXMTexture *texture)
{
    for (Csm::csmUint32 i = 0; i < _textures.GetSize(); i++)
    {
        if (_textures[i]->texture != texture)
        {
            continue;
        }
        LAppGXM::GetInstance()->UnloadGxtTexture(_textures[i]->texture);
        delete _textures[i];
        _textures.Remove(i);
        break;
    }
}

void LAppTextureManager::ReleaseTexture(std::string fileName)
{
    for (Csm::csmUint32 i = 0; i < _textures.GetSize(); i++)
    {
        if (_textures[i]->fileName == fileName)
        {
            LAppGXM::GetInstance()->UnloadGxtTexture(_textures[i]->texture);
            delete _textures[i];
            _textures.Remove(i);
            break;
        }
    }
}

LAppTextureManager::TextureInfo* LAppTextureManager::GetTextureInfoByGXMTexture(LAppGXM::GXMTexture *texture) const
{
    for (Csm::csmUint32 i = 0; i < _textures.GetSize(); i++)
    {
        if (_textures[i]->texture == texture)
        {
            return _textures[i];
        }
    }

    return NULL;
}
